<?php
/*
  Plugin Name: EasyPay Payment Gateway
  Plugin URI: 
  Description: Allows you to use EasyPay payment gateway with the WooCommerce plugin.
  Version: 0.9
  Author: Andrei Yudenko
  Author URI: http://easypay.by
 */

if (!defined('ABSPATH')) exit; // Exit if accessed directly

/* Add a custom payment class to WC
  ------------------------------------------------------------ */

add_action('plugins_loaded', 'load_easypay', 0);

function load_easypay() {
	if (!class_exists('WC_Payment_Gateway')) {
		return; // if the WC payment gateway class is not available, do nothing
	}
	if(class_exists('WC_EASYPAY')) {
		return;
	}

class WC_EASYPAY extends WC_Payment_Gateway {

	public function __construct() {
		$plugin_dir = plugin_dir_url(__FILE__);
		global $woocommerce;
		$this->id      = "easypay";
		$this->icon    = "https://ssl.easypay.by/imgs/icons/easypay_icon.gif";
		$this->liveurl = 'https://ssl.easypay.by/weborder/?EP_Module=woocommerce_2_1';

		// Load the settings
		$this->init_form_fields();
		$this->init_settings();

		// Define user set variables
		$this->title       = $this->get_option('title');
		$this->mer_no      = $this->get_option('mer_no');
		$this->web_key     = $this->get_option('web_key');
		$this->debug       = $this->get_option('debug');
		$this->description = $this->get_option('description');
		$this->expires     = $this->get_option('expires');

		$this->log = $woocommerce->logger();

		// Actions
		add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));

		// сохранение данных в админке
		add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

		// вызов функции для обработки урлов возврата
		add_action('woocommerce_api_wc_' . $this->id, array($this, 'check_response'));
	}

	/**
	* Admin Panel Options 
	* - Options for bits like 'title' and availability on a country-by-country basis
	*
	* @since 0.1
	**/
	public function admin_options() {
		?>
		<h3><?php _e('EASYPAY', 'woocommerce'); ?></h3>
		<p><?php _e('Настройка оплаты Visa, MasterCard', 'woocommerce'); ?></p>
		<table class="form-table">
		<?php
				// Generate the HTML For the settings form.
				$this->generate_settings_html();
	?>
	</table><!--/.form-table-->
		<?php
	} // End admin_options()

  /**
  * Initialise Gateway Settings Form Fields
  *
  * @access public
  * @return void
  */
	function init_form_fields(){
		$this->form_fields = array(
			'enabled' => array(
				'title'   => __('Включить/Выключить', 'woocommerce'),
				'type'    => 'checkbox',
				'label'   => __('Включен', 'woocommerce'),
				'default' => 'no'
			),
			'title' => array(
				'title'       => __('Название', 'woocommerce'),
				'type'        => 'text', 
				'description' => __('Это название, которое пользователь видит во время проверки.', 'woocommerce'), 
				'default'     => __('EasyPay', 'woocommerce')
			),
			'mer_no' => array(
				'title'       => __('Номер Поставщика', 'woocommerce'),
				'type'        => 'text',
				'description' => __('Пожалуйста введите номер Поставщика', 'woocommerce'),
				'default'     => ''
			),
			'web_key' => array(
				'title'       => __('Ключ для подписи счетов', 'woocommerce'),
				'type'        => 'text',
				'description' => __('Пожалуйста введите ключ для подписи счетов.', 'woocommerce'),
				'default'     => ''
			),
			'expires' => array(
				'title'       => __('Срок действия счета', 'woocommerce'),
				'type'        => 'text',
				'description' => __('Пожалуйста введите срок действия счетов.', 'woocommerce'),
				'default'     => '3'
			),
			'debug' => array(
				'title'       => __('Режим отладки', 'woocommerce'),
				'type'        => 'checkbox', 
				'label'       => __('Включен', 'woocommerce'),
				'description' => __('В этом режиме выводится подробная информация о передаваемых данных.', 'woocommerce'),
				'default'     => 'no'
			),
			'description' => array(
				'title'       => __('Описание метода оплаты', 'woocommerce'),
				'type'        => 'textarea',
				'description' => __('Описанием метода оплаты которое клиент будет видеть на вашем сайте.', 'woocommerce'),
				'default'     => 'Оплата Visa, MasterCard'
			),
		);
	}

	/**
	* Generate the dibs button link
	**/
	public function generate_form($order_id){
		global $woocommerce;
		$order = new WC_Order($order_id);
		$action_adr = $this->liveurl;
		$out_summ = $order->order_total;
		$hash = md5($this->mer_no . $this->web_key . $order_id . $out_summ);
		$this->debug = ($this->debug == 'yes') ? 1 : 0;
		$args = array(
			'EP_MerNo'       => $this->mer_no,
			'EP_OrderNo'     => $order_id,
			'EP_Sum'         => $out_summ,
			'EP_Expires'     => $this->expires,
			'EP_Comment'     => '',
			'EP_OrderInfo'   => '',
			'EP_Hash'        => $hash,
			'EP_Success_URL' => site_url() . '/?wc-api=wc_easypay&result=success&EP_OrderNo=' . $order_id,
			'EP_Cancel_URL'  => site_url() . '/?wc-api=wc_easypay&result=fail&EP_OrderNo='    . $order_id,
			'EP_Debug'       => $this->debug,
			'EP_URL_Type'    => 'link',
		);
		$args_array = array();
		foreach ($args as $key => $value){
			$args_array[] = '<input type="hidden" name="' . esc_attr($key) . '" value="' . $value . '" />';
		}
		return
			'<form action="' . esc_url($action_adr) . '" method="POST" id="easypay_payment_form">' . "\n".
			implode("\n", $args_array) . '<input type="submit" class="button alt" id="submit_easypay_payment_form" value="' . __('Оплатить', 'woocommerce') . '" /> <a class="button cancel" href="' . $order->get_cancel_order_url() . '">' . __('Отказаться от оплаты & вернуться в корзину', 'woocommerce') . '</a>' . "\n" . '</form>';
	}

	/**
	 * Process the payment and return the result
	 **/
	function process_payment($order_id){
		$order = new WC_Order($order_id);
		return array(
			'result' => 'success',
			'redirect'	=> add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay'))))
		);
	}

	/**
	* receipt_page
	**/
	function receipt_page($order){
		echo '<p>' . __('Спасибо за Ваш заказ, пожалуйста, нажмите кнопку ниже, чтобы оплатить.', 'woocommerce') . '</p>';
		echo $this->generate_form($order);
	}

	/**
	* Check Response
	**/
	function check_response() {
		global $woocommerce;
		if (isset($_GET['result'])) {
			if ($_GET['result'] == 'success') {
				@ob_clean();
				$_GET = stripslashes_deep($_GET);
				$order_mer_code = $_GET['EP_OrderNo'];
				$order = new WC_Order($order_mer_code);
				WC()->cart->empty_cart();
				wp_redirect($this->get_return_url($order));
			} else if ($_GET['result'] == 'fail') {
				$order_mer_code = $_GET['EP_OrderNo'];
				$order = new WC_Order($order_mer_code);
				wp_redirect(WC()->cart->get_cart_url());
				exit;
			} else if ($_GET['result'] == 'notify') {
				$order_mer_code = isset($_POST['order_mer_code']) ? $_POST['order_mer_code'] : '';
				$sum = isset($_POST['sum']) ? $_POST['sum'] : '';
				$mer_no = isset($_POST['mer_no']) ? $_POST['mer_no'] : '';
				$card = isset($_POST['card']) ? $_POST['card'] : '';
				$notify_signature = isset($_POST['notify_signature']) ? $_POST['notify_signature'] : '';
				$purch_date = isset($_POST['purch_date']) ? $_POST['purch_date'] : '';
                                $web_key = $this->web_key;
				$sign = md5($order_mer_code . $sum . $mer_no . $card . $purch_date . $web_key);
                                
                $order = new WC_Order($order_mer_code);

				if ($notify_signature == $sign) { // проверка правильности подписи
                        $order->update_status('processing', __('Счет успешно оплачен', 'woocommerce'));
                        $status = 'OK | payment received';
                        header("HTTP/1.0 200 OK");
                        print $status;
                                   
				} else { //подписи неравны
                        $status = 'FAILED | wrong notify signature'; 
                        header("HTTP/1.0 400 Bad Request");
                        print $status;
                }
				exit;
			}
		}
	}
}

/**
 * Add the gateway to WooCommerce
 **/
function add_easypay_gateway($methods) {
	$methods[] = 'WC_EASYPAY';
	return $methods;
}
add_filter('woocommerce_payment_gateways', 'add_easypay_gateway');
}
?>